% Compute the restoring force vectors
% by which the individual gcells act on the nodes.
% Return an array of elevec objects so they may be assembled.
%    Call as
% evs = restoring_force(feb, geom, u)
%     geom     - geometry field
%     u        - displacement field
%
% Note: This method *UPDATES* the state of the feblock object.  In
%       particular, the material state gets updated.  If this outdated
%       state should be preserved, the feblock must be assigned to some
%       variable on return; otherwise the first return value must be ignored!
%
function [self,evs] = restoring_force(self, geom, u)
gcells = get(self.feblock,'gcells');
ngcells = length(gcells);
nfens = get(gcells(1),'nfens');
dim = get(geom,'dim');
% Pre-allocate the element matrices
evs(1:ngcells) = deal(elevec);
% Integration rule
integration_rule = get(self.feblock, 'integration_rule');
pc = get(integration_rule, 'param_coords');
w  = get(integration_rule, 'weights');
npts_per_gcell = length(pc); % number of integration point
% Material
mat = get(self.feblock, 'mater');
matstates = get(self.feblock, 'matstates');
% Now loop over all gcells in the block
for i=1:ngcells
    conn = get(gcells(i), 'conn'); % connectivity
    X = gather(geom, conn, 'values', 'noreshape'); % reference coordinates
    U = gather(u, conn, 'values', 'noreshape'); % displacement
    x1 = X + U; % displaced coordinates
    Fe = zeros(dim*nfens,1); % element vector
    % Loop over all integration points
    for j=1:npts_per_gcell
        Nder = Ndermat_param (gcells(i), pc(j,:));
        [NderX,J] = Ndermat_spatial (gcells(i), Nder, X);
        F1 = def_grad(gcells(i), NderX, x1);
        B = Blmat (gcells(i), NderX);
        [sigma,matstates{i,j}] = update(mat, matstates{i,j}, struct ('F1',F1));
        Fe = Fe - B'* sigma * J * w(j); % note the sign
    end
    evs(i) = set(evs(i), 'vec', Fe);
    evs(i) = set(evs(i), 'eqnums', gather(u, conn, 'eqnums'));
end
self.feblock = set(self.feblock,'matstates',matstates);
return;
